@@ -739,6 +739,26 @@ def _from_stream(
     return inst
 
 
+def ncities_from_tsplib_name(name: str) -> int:
+    """
+    Compute the instance scale from the instance name.
+
+    :param name: the instance name
+    :return: the instance scale
+    """
+    if name == "kro124p":
+        return 100
+    if name == "ry48p":
+        return 48
+    idx: int = len(name)
+    while name[idx - 1] in "0123456789":
+        idx -= 1
+    scale: Final[int] = int(name[idx:])
+    if name.startswith("ftv"):
+        return scale + 1
+    return scale
+
+
 class Instance(Component, np.ndarray):
     """An instance of the Traveling Salesperson Problem."""
 
@@ -754,7 +774,8 @@ class Instance(Component, np.ndarray):
     is_symmetric: bool
 
     def __new__(cls, name: str, tour_length_lower_bound: int,
-                matrix: np.ndarray) -> "Instance":
+                matrix: np.ndarray,
+                upper_bound_range_multiplier: int = 1) -> "Instance":
         """
         Create an instance of the Traveling Salesperson Problem.
 
@@ -762,6 +783,8 @@ class Instance(Component, np.ndarray):
         :param name: the name of the instance
         :param tour_length_lower_bound: the lower bound of the tour length
         :param matrix: the matrix with the data (will be copied)
+        :param upper_bound_range_multiplier: a multiplier for the upper bound
+            to fix the data range, by default `1`
         """
         use_name: Final[str] = sanitize_name(name)
         if name != use_name:
@@ -799,11 +822,11 @@ class Instance(Component, np.ndarray):
                         raise ValueError(
                             f"if i=j={i}, then dist must be zero "
                             f"but is {dist}.")
-                else:
-                    if dist > farthest_neighbor:
-                        farthest_neighbor = dist
-                    if dist < nearest_neighbor:
-                        nearest_neighbor = dist
+                    continue
+                if dist > farthest_neighbor:
+                    farthest_neighbor = dist
+                if dist < nearest_neighbor:
+                    nearest_neighbor = dist
                 if dist != matrix[j, i]:
                     is_symmetric = False
             if farthest_neighbor <= 0:
@@ -818,11 +841,20 @@ class Instance(Component, np.ndarray):
         check_int_range(upper_bound, "upper_bound",
                         tour_length_lower_bound, 1_000_000_000_000_001)
 
-        # create the object
+        # create the object and ensure that the backing integer type is
+        # large enough to accommodate all possible tour lengths
+        limit: Final[int] = (check_int_range(
+            upper_bound_range_multiplier, "upper_bound_range_multiplier", 1)
+            * max(upper_bound, n_cities))
         obj: Final[Instance] = super().__new__(
             cls, use_shape, int_range_to_dtype(
-                min_value=0, max_value=max(upper_bound, n_cities)))
+                min_value=-limit, max_value=limit))
         np.copyto(obj, matrix, "unsafe")
+        for i in range(n_cities):
+            for j in range(n_cities):
+                if obj[i, j] != matrix[i, j]:
+                    raise ValueError(
+                        f"error when copying: {obj[i, j]} != {matrix[i, j]}")
 
         #: the name of the instance
         obj.name = use_name
@@ -881,6 +913,11 @@ symmetric: T@dtype: i@END_I'
         :param lower_bound_getter: a function returning a lower bound for an
             instance name, or `None` to use a simple lower bound approximation
         :return: the instance
+
+        >>> from os.path import dirname
+        >>> inst = Instance.from_file(dirname(__file__) + "/tsplib/br17.atsp")
+        >>> inst.name
+        'br17'
         """
         file: Final[Path] = Path.file(path)
         with file.open_for_read() as stream:
@@ -893,7 +930,7 @@ symmetric: T@dtype: i@END_I'
     @staticmethod
     def from_resource(name: str) -> "Instance":
         """
-        Load an instance from a resource.
+        Load a TSP instance from a resource.
 
         :param name: the name string
         :return: the instance
@@ -912,19 +949,24 @@ symmetric: T@dtype: i@END_I'
         suffix: Final[str] = ".tsp" if is_symmetric else ".atsp"
         with open_resource_stream(f"{name}{suffix}") as stream:
             inst: Final[Instance] = _from_stream(stream)
-            if inst.name != name:
-                raise ValueError(f"got {inst.name!r} for instance {name!r}?")
-            if is_symmetric and (not inst.is_symmetric):
-                raise ValueError(f"{name!r} should be symmetric but is not?")
-            if inst.n_cities <= 1000:
-                setattr(container, inst_attr, inst)
-            return inst
+
+        if inst.name != name:
+            raise ValueError(f"got {inst.name!r} for instance {name!r}?")
+        sc: int = ncities_from_tsplib_name(name)
+        if sc != inst.n_cities:
+            raise ValueError(f"expected to get n_cities={sc} from name "
+                             f"{name!r} but got {inst.n_cities}.")
+        if is_symmetric and (not inst.is_symmetric):
+            raise ValueError(f"{name!r} should be symmetric but is not?")
+        if inst.n_cities <= 1000:
+            setattr(container, inst_attr, inst)
+        return inst
 
     @staticmethod
     def list_resources(symmetric: bool = True,
                        asymmetric: bool = True) -> tuple[str, ...]:
         """
-        Get a tuple of all the instances available as resource.
+        Get a tuple of all the TSP instances available as resource.
 
         :param symmetric: include the symmetric instances
         :param asymmetric: include the asymmetric instances
